@@ -14,7 +14,9 @@
    # check for character data
    if(is.character(x)) stop("spatial sign is not defined for character data")
    xNames <- dimnames(x)
+   p <- ncol(x)
    tmp <- t(apply(x, 1, spatialSign.default))
+   if(p == 1 & nrow(tmp) == 1) tmp <- t(tmp)
    dimnames(tmp) <- xNames
    tmp
 }
@@ -24,8 +26,9 @@
    if(any(apply(x, 2, function(data) is.character(data) | is.factor(data))))
       stop("spatial sign is not defined for character or factor data")
    xNames <- dimnames(x)
-   tmp <- t(apply(x, 1, spatialSign.default))
-   tmp <- as.data.frame(tmp)
+   x <- as.matrix(x)
+   if(!is.numeric(x)) stop("a character matrix was the result of as.matrix")
+   tmp <- spatialSign(x)
    dimnames(tmp) <- xNames
    tmp
 }
