@@ -43,7 +43,6 @@ expandParameters <- function(fixed, seq)
 
 nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing = FALSE, ...)
 {
-  library(caret)
   loadNamespace("caret")
   ppp <- list(options = ppOpts)
   ppp <- c(ppp, ctrl$preProcOptions)
@@ -71,7 +70,7 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
   testing <- FALSE
   if(!(length(ctrl$seeds) == 1 && is.na(ctrl$seeds))) set.seed(ctrl$seeds[[iter]][parm])
   
-  library(caret)
+  loadNamespace("caret")
   if(ctrl$verboseIter) progress(printed[parm,,drop = FALSE],
                                 names(resampleIndex), iter)
   
@@ -234,17 +233,19 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
     
     ## collate the predicitons across all the sub-models
     predicted <- lapply(predicted,
-                        function(x, y, wts, lv) {
+                        function(x, y, wts, lv, rows) {
                           if(!is.factor(x) & is.character(x)) x <- factor(as.character(x), levels = lv)
                           out <- data.frame(pred = x, obs = y, stringsAsFactors = FALSE)
                           if(!is.null(wts)) out$weights <- wts
+                          out$rowIndex <- rows
                           out
                         },
                         y = y[holdoutIndex],
                         wts = wts[holdoutIndex],
-                        lv = lev)
+                        lv = lev,
+                        rows = holdoutIndex)
     if(testing) print(head(predicted))
-    
+
     ## same for the class probabilities
     if(ctrl$classProbs)
     {
@@ -292,6 +293,7 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
     names(tmp)[1] <- "pred"
     if(!is.null(wts)) tmp$weights <- wts[holdoutIndex]
     if(ctrl$classProbs) tmp <- cbind(tmp, probValues)
+    tmp$rowIndex <- holdoutIndex
     
     if(ctrl$savePredictions)
     {
@@ -303,6 +305,7 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
     } else tmpPred <- NULL
 
     ##################################
+    
     thisResample <- ctrl$summaryFunction(tmp,
                                          lev = lev,
                                          model = method)
@@ -368,7 +371,6 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
 
 looTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing = FALSE, ...)
 {
-  library(caret)
   loadNamespace("caret")
   
   ppp <- list(options = ppOpts)
@@ -387,7 +389,7 @@ looTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing
       
       if(!(length(ctrl$seeds) == 1 && is.na(ctrl$seeds))) set.seed(ctrl$seeds[[iter]][parm])
       if(testing) cat("after loops\n")
-      library(caret)
+      loadNamespace("caret")
       if(ctrl$verboseIter) progress(printed[parm,,drop = FALSE],
                                     names(ctrl$index), iter, TRUE)
       if(is.null(info$submodels[[parm]]) || nrow(info$submodels[[parm]]) > 0) {
@@ -459,15 +461,17 @@ looTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing
       {
         ## collate the predictions across all the sub-models
         predicted <- lapply(predicted,
-                            function(x, y, wts, lv) {
+                            function(x, y, wts, lv, rows) {
                               if(!is.factor(x) & is.character(x)) x <- factor(as.character(x), levels = lv)
                               out <- data.frame(pred = x, obs = y, stringsAsFactors = FALSE)
                               if(!is.null(wts)) out$weights <- wts
+                              out$rowIndex <- rows
                               out
                             },
                             y = y[holdoutIndex],
                             wts = wts[holdoutIndex],
-                            lv = lev)
+                            lv = lev,
+                            rows = seq(along = y)[holdoutIndex])
         if(testing) print(head(predicted))
         
         ## same for the class probabilities
@@ -481,7 +485,6 @@ looTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing
         predicted <- cbind(predicted, allParam)
         ## if saveDetails then save and export 'predicted'
       } else {
-        
         if(is.factor(y)) predicted <- factor(as.character(predicted),
                                              levels = lev)
         predicted <-  data.frame(pred = predicted,
@@ -489,6 +492,7 @@ looTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing
                                  stringsAsFactors = FALSE)
         if(!is.null(wts)) predicted$weights <- wts[holdoutIndex]
         if(ctrl$classProbs) predicted <- cbind(predicted, probValues)
+        predicted$rowIndex <- seq(along = y)[holdoutIndex]
         predicted <- cbind(predicted, info$loop[parm,,drop = FALSE])
         
       }
@@ -508,7 +512,7 @@ looTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing
 
 oobTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing = FALSE, ...)
 {
-  library(caret)
+  loadNamespace("caret")
   ppp <- list(options = ppOpts)
   ppp <- c(ppp, ctrl$preProcOptions)
   printed <- format(info$loop)
@@ -518,7 +522,7 @@ oobTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing
   if(!is.null(method$library)) pkgs <- c(pkgs, method$library)
   result <- foreach(parm = 1:nrow(info$loop), .packages = pkgs, .combine = "rbind") %op%
 {
-  library(caret)
+  loadNamespace("caret")
   if(ctrl$verboseIter) progress(printed[parm,,drop = FALSE], "", 1, TRUE)
   
   mod <- createModel(x = x,
@@ -548,7 +552,7 @@ oobTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing
 
 nominalSbfWorkflow <- function(x, y, ppOpts, ctrl, lev, ...)
 {
-  library(caret)
+  loadNamespace("caret")
   ppp <- list(options = ppOpts)
   ppp <- c(ppp, ctrl$preProcOptions)
   
@@ -563,7 +567,7 @@ nominalSbfWorkflow <- function(x, y, ppOpts, ctrl, lev, ...)
 {
   if(!(length(ctrl$seeds) == 1 && is.na(ctrl$seeds))) set.seed(ctrl$seeds[iter])
   
-  library(caret)
+  loadNamespace("caret")
   
   if(names(resampleIndex)[iter] != "AllData") {
     modelIndex <- resampleIndex[[iter]]
@@ -623,7 +627,7 @@ nominalSbfWorkflow <- function(x, y, ppOpts, ctrl, lev, ...)
 
 looSbfWorkflow <- function(x, y, ppOpts, ctrl, lev, ...)
 {
-  library(caret)
+  loadNamespace("caret")
   ppp <- list(options = ppOpts)
   ppp <- c(ppp, ctrl$preProcOptions)
   
@@ -636,7 +640,7 @@ looSbfWorkflow <- function(x, y, ppOpts, ctrl, lev, ...)
 {
   if(!(length(ctrl$seeds) == 1 && is.na(ctrl$seeds))) set.seed(ctrl$seeds[iter])
   
-  library(caret)
+  loadNamespace("caret")
   
   modelIndex <- resampleIndex[[iter]]
   holdoutIndex <- -unique(resampleIndex[[iter]])
@@ -661,7 +665,7 @@ looSbfWorkflow <- function(x, y, ppOpts, ctrl, lev, ...)
 
 nominalRfeWorkflow <- function(x, y, sizes, ppOpts, ctrl, lev, ...)
 {
-  library(caret)
+  loadNamespace("caret")
   ppp <- list(options = ppOpts)
   ppp <- c(ppp, ctrl$preProcOptions)
   
@@ -674,7 +678,7 @@ nominalRfeWorkflow <- function(x, y, sizes, ppOpts, ctrl, lev, ...)
   `%op%` <- getOper(ctrl$allowParallel && getDoParWorkers() > 1)
   result <- foreach(iter = seq(along = resampleIndex), .combine = "c", .verbose = FALSE, .packages = c("methods", "caret", "plyr"), .errorhandling = "stop") %op%
 {
-  library(caret)
+  loadNamespace("caret")
   
   if(names(resampleIndex)[iter] != "AllData") {
     modelIndex <- resampleIndex[[iter]]
@@ -752,7 +756,7 @@ nominalRfeWorkflow <- function(x, y, sizes, ppOpts, ctrl, lev, ...)
 
 looRfeWorkflow <- function(x, y, sizes, ppOpts, ctrl, lev, ...)
 {
-  library(caret)
+  loadNamespace("caret")
   ppp <- list(options = ppOpts)
   ppp <- c(ppp, ctrl$preProcOptions)
   
@@ -760,7 +764,7 @@ looRfeWorkflow <- function(x, y, sizes, ppOpts, ctrl, lev, ...)
   `%op%` <- getOper(ctrl$allowParallel && getDoParWorkers() > 1)
   result <- foreach(iter = seq(along = resampleIndex), .combine = "c", .verbose = FALSE, .packages = c("methods", "caret"), .errorhandling = "stop") %op%
 {
-  library(caret)
+  loadNamespace("caret")
   
   modelIndex <- resampleIndex[[iter]]
   holdoutIndex <- -unique(resampleIndex[[iter]])
